@@ -1,7 +1,6 @@
 // SPDX-License-Identifier: LGPL-2.1-or-later
 // Copyright (c) 2012-2014 Monty Program Ab
-// Copyright (c) 2015-2021 MariaDB Corporation Ab
-
+// Copyright (c) 2015-2023 MariaDB Corporation Ab
 package org.mariadb.jdbc.integration;
 
 import static org.junit.jupiter.api.Assertions.*;
@@ -32,6 +31,67 @@ public class BatchTest extends Common {
     stmt.execute("DROP TABLE IF EXISTS BatchTest");
   }
 
+  @Test
+  public void batchClear() throws SQLException {
+    Statement stmt = sharedConn.createStatement();
+    stmt.execute("DROP TABLE IF EXISTS batchClear");
+    stmt.execute("CREATE TABLE batchClear(c0 VARCHAR(16))");
+    try (PreparedStatement prep =
+        sharedConn.prepareStatement("INSERT INTO batchClear VALUES (?)")) {
+      prep.setString(1, "1");
+      prep.addBatch();
+
+      prep.setString(1, "2");
+      prep.addBatch();
+
+      prep.setString(1, "3");
+      prep.addBatch();
+
+      prep.executeBatch();
+
+      prep.setString(1, "4");
+      prep.addBatch();
+      prep.clearBatch();
+
+      prep.setString(1, "5");
+      prep.addBatch();
+
+      prep.executeBatch();
+    }
+    ResultSet rs = stmt.executeQuery("SELECT * FROM batchClear");
+    Assertions.assertTrue(rs.next());
+    assertEquals("1", rs.getString(1));
+    Assertions.assertTrue(rs.next());
+    assertEquals("2", rs.getString(1));
+    Assertions.assertTrue(rs.next());
+    assertEquals("3", rs.getString(1));
+    Assertions.assertTrue(rs.next());
+    assertEquals("5", rs.getString(1));
+  }
+
+  @Test
+  public void batchError() throws SQLException {
+    Statement stmt = sharedConn.createStatement();
+    stmt.execute("DROP TABLE IF EXISTS t1");
+    stmt.execute("CREATE TABLE t1(c0 DATE UNIQUE PRIMARY KEY NOT NULL)");
+
+    stmt.addBatch("INSERT INTO t1 VALUES ('2006-04-01')");
+    stmt.addBatch("INSERT INTO t1 VALUES ('2006-04-01')");
+    stmt.addBatch("INSERT INTO t1 VALUES ('2019-04-11')");
+    stmt.addBatch("INSERT INTO t1 VALUES ('2006-04-01')");
+    stmt.addBatch("INSERT INTO t1 VALUES ('2020-04-11')");
+    try {
+      stmt.executeBatch();
+      fail();
+    } catch (BatchUpdateException e) {
+      assertTrue(e.getMessage().contains("Duplicate entry"));
+      assertEquals(5, e.getUpdateCounts().length);
+      assertArrayEquals(
+          new int[] {1, java.sql.Statement.EXECUTE_FAILED, 1, java.sql.Statement.EXECUTE_FAILED, 1},
+          e.getUpdateCounts());
+    }
+  }
+
   @Test
   public void wrongParameter() throws SQLException {
     try (Connection con = createCon("&useServerPrepStmts=false")) {
@@ -60,7 +120,8 @@ public class BatchTest extends Common {
             e.getMessage().contains("Parameter at position 2 is not set")
                 || e.getMessage()
                     .contains(
-                        "batch set of parameters differ from previous set. All parameters must be set"));
+                        "batch set of parameters differ from previous set. All parameters must be"
+                            + " set"));
       }
 
       prep.setInt(1, 5);
@@ -80,36 +141,56 @@ public class BatchTest extends Common {
 
   @Test
   public void differentParameterType() throws SQLException {
-    try (Connection con = createCon("&useServerPrepStmts=false&useBulkStmts=false")) {
+    try (Connection con = createCon("&useServerPrepStmts=false&useBulkStmtsForInserts=false")) {
       differentParameterType(con, false);
     }
-    try (Connection con = createCon("&useServerPrepStmts=false&useBulkStmts=true")) {
+    try (Connection con =
+        createCon("&useServerPrepStmts=false&useBulkStmts&useBulkStmtsForInserts")) {
       differentParameterType(con, isMariaDBServer() && !isXpand());
     }
     try (Connection con =
-        createCon("&useServerPrepStmts=false&useBulkStmts=true&disablePipeline")) {
+        createCon(
+            "&useServerPrepStmts=false&useBulkStmtsForInserts&useBulkStmts&disablePipeline")) {
       differentParameterType(con, isMariaDBServer() && !isXpand());
     }
-    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts=false")) {
+    try (Connection con = createCon("&useServerPrepStmts&useBulkStmtsForInserts=false")) {
       differentParameterType(con, false);
     }
-    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts&allowLocalInfile=false")) {
+    try (Connection con =
+        createCon("&useServerPrepStmts&useBulkStmtsForInserts&allowLocalInfile=false")) {
+      differentParameterType(con, false);
+    }
+    try (Connection con =
+        createCon(
+            "&useServerPrepStmts&useBulkStmts&useBulkStmtsForInserts&allowLocalInfile=false")) {
+      differentParameterType(con, isMariaDBServer() && !isXpand());
+    }
+    try (Connection con = createCon("&useServerPrepStmts=false&useBulkStmts&allowLocalInfile")) {
       differentParameterType(con, isMariaDBServer() && !isXpand());
     }
     try (Connection con = createCon("&useServerPrepStmts=false&allowLocalInfile")) {
-      differentParameterType(con, isMariaDBServer() && !isXpand());
-    }
-    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts=false")) {
       differentParameterType(con, false);
     }
-    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts")) {
-      differentParameterType(con, isMariaDBServer() && !isXpand());
+    try (Connection con = createCon("&useServerPrepStmts&useBulkStmtsForInserts=false")) {
+      differentParameterType(con, false);
     }
-    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts&allowLocalInfile=false")) {
+    try (Connection con = createCon("&useServerPrepStmts&useBulkStmtsForInserts")) {
+      differentParameterType(con, false);
+    }
+    try (Connection con = createCon("&useServerPrepStmts&useBulkStmts&useBulkStmtsForInserts")) {
       differentParameterType(con, isMariaDBServer() && !isXpand());
     }
     try (Connection con =
-        createCon("&useServerPrepStmts&useBulkStmts=false&disablePipeline=true")) {
+        createCon(
+            "&useServerPrepStmts&useBulkStmts&useBulkStmtsForInserts&allowLocalInfile=false")) {
+      differentParameterType(con, isMariaDBServer() && !isXpand());
+    }
+    try (Connection con =
+        createCon("&useServerPrepStmts&useBulkStmtsForInserts&allowLocalInfile=false")) {
+      differentParameterType(con, false);
+    }
+    try (Connection con =
+        createCon("&useServerPrepStmts&useBulkStmtsForInserts=false&disablePipeline=true")) {
       differentParameterType(con, false);
     }
   }
@@ -118,6 +199,7 @@ public class BatchTest extends Common {
       throws SQLException {
     Statement stmt = con.createStatement();
     stmt.execute("TRUNCATE BatchTest");
+    stmt.execute("START TRANSACTION"); // if MAXSCALE ensure using WRITER
     try (PreparedStatement prep =
         con.prepareStatement("INSERT INTO BatchTest(t1, t2) VALUES (?,?)")) {
       prep.setInt(1, 1);
@@ -132,15 +214,9 @@ public class BatchTest extends Common {
       prep.addBatch();
       int[] res = prep.executeBatch();
       assertEquals(3, res.length);
-      if (expectSuccessUnknown) {
-        assertEquals(Statement.SUCCESS_NO_INFO, res[0]);
-        assertEquals(Statement.SUCCESS_NO_INFO, res[1]);
-        assertEquals(Statement.SUCCESS_NO_INFO, res[2]);
-      } else {
-        assertEquals(1, res[0]);
-        assertEquals(1, res[1]);
-        assertEquals(1, res[2]);
-      }
+      assertEquals(1, res[0]);
+      assertEquals(1, res[1]);
+      assertEquals(1, res[2]);
     }
     ResultSet rs = stmt.executeQuery("SELECT * FROM BatchTest");
     assertTrue(rs.next());
@@ -166,13 +242,8 @@ public class BatchTest extends Common {
       prep.addBatch();
       int[] res = prep.executeBatch();
       assertEquals(2, res.length);
-      if (expectSuccessUnknown) {
-        assertEquals(Statement.SUCCESS_NO_INFO, res[0]);
-        assertEquals(Statement.SUCCESS_NO_INFO, res[1]);
-      } else {
-        assertEquals(1, res[0]);
-        assertEquals(1, res[1]);
-      }
+      assertEquals(1, res[0]);
+      assertEquals(1, res[1]);
     }
     rs = stmt.executeQuery("SELECT * FROM BatchTest");
     assertTrue(rs.next());
@@ -241,21 +312,44 @@ public class BatchTest extends Common {
     assertEquals(2, rs.getInt(1));
     assertEquals("2", rs.getString(2));
     assertFalse(rs.next());
+
+    try (PreparedStatement prep =
+        con.prepareStatement("UPDATE BatchTest SET t1=t1+10 WHERE t1=?")) {
+      prep.setInt(1, 1);
+      prep.addBatch();
+
+      prep.setInt(1, 2);
+      prep.addBatch();
+      int[] res = prep.executeBatch();
+      if (expectSuccessUnknown) {
+        assertEquals(Statement.SUCCESS_NO_INFO, res[0]);
+        assertEquals(Statement.SUCCESS_NO_INFO, res[1]);
+      } else {
+        assertEquals(1, res[0]);
+        assertEquals(1, res[1]);
+      }
+    }
+    con.rollback();
   }
 
   @Test
   public void largeBatch() throws SQLException {
-    for (int i = 0; i < 32; i++) {
+    for (int i = 0; i < 64; i++) {
       boolean useServerPrepStmts = (i & 2) > 0;
       boolean useBulkStmts = (i & 4) > 0;
       boolean allowLocalInfile = (i & 8) > 0;
       boolean useCompression = (i & 16) > 0;
+      boolean useBulkStmtsForInserts = (i & 32) > 0;
 
-      try (Connection con =
-          createCon(
-              String.format(
-                  "&useServerPrepStmts=%s&useBulkStmts=%s&allowLocalInfile=%s&useCompression=%s",
-                  useServerPrepStmts, useBulkStmts, allowLocalInfile, useCompression))) {
+      String confString =
+          String.format(
+              "&useServerPrepStmts=%s&useBulkStmts=%s&allowLocalInfile=%s&useCompression=%s&useBulkStmtsForInserts=%s",
+              useServerPrepStmts,
+              useBulkStmts,
+              allowLocalInfile,
+              useCompression,
+              useBulkStmtsForInserts);
+      try (Connection con = createCon(confString)) {
         largeBatch(con);
       }
     }
@@ -420,29 +514,6 @@ public class BatchTest extends Common {
     }
   }
 
-  private class TimestampCal {
-    private Timestamp val;
-    private int id;
-
-    public TimestampCal(Timestamp val, int id) {
-      this.val = val;
-      this.id = id;
-    }
-
-    public Timestamp getVal() {
-      return val;
-    }
-
-    public int getId() {
-      return id;
-    }
-
-    @Override
-    public String toString() {
-      return "TimestampCal{" + "val=" + val + ", id=" + id + '}';
-    }
-  }
-
   @Test
   public void ensureCalendarSync() throws SQLException {
     Assumptions.assumeTrue(isMariaDBServer() && !isXpand());
@@ -489,4 +560,27 @@ public class BatchTest extends Common {
       return -1;
     }
   }
+
+  private class TimestampCal {
+    private final Timestamp val;
+    private final int id;
+
+    public TimestampCal(Timestamp val, int id) {
+      this.val = val;
+      this.id = id;
+    }
+
+    public Timestamp getVal() {
+      return val;
+    }
+
+    public int getId() {
+      return id;
+    }
+
+    @Override
+    public String toString() {
+      return "TimestampCal{" + "val=" + val + ", id=" + id + '}';
+    }
+  }
 }
